﻿using System.Collections.Generic;
using System.Linq;
using System.Web;
using EPiServer.Cms.Shell;
using EPiServer.Cms.Shell.UI.Rest.ContentQuery;
using EPiServer.Core;
using EPiServer.ServiceLocation;
using EPiServer.Shell.Search;
using EPiServer.Shell.ViewComposition;

namespace EPiServer.Templates.Alloy.UIExtensions.CustomSearch
{
    [ServiceConfiguration(typeof(IContentQuery))]
    public class CustomSearchQuery : ContentQueryBase
    {
        private readonly IContentRepository _contentRepository;
        private readonly SearchProvidersManager _searchProvidersManager;
        private readonly LanguageSelectorFactory _languageSelectorFactory;

        public CustomSearchQuery(
            IContentQueryHelper queryHelper,
            IContentRepository contentRepository,
            SearchProvidersManager searchProvidersManager,
            LanguageSelectorFactory languageSelectorFactory)
            : base(queryHelper)
        {
            _contentRepository = contentRepository;
            _searchProvidersManager = searchProvidersManager;
            _languageSelectorFactory = languageSelectorFactory;
        }

        /// <summary>
        /// The key to trigger this query.
        /// </summary>
        public override string Name
        {
            get { return "CustomQuery"; }
        }

        protected override IEnumerable<IContent> GetContent(ContentQueryParameters parameters)
        {
            var queryText = HttpUtility.HtmlDecode(parameters.AllParameters["queryText"]);
            var searchQuery = new Query(queryText);

            var contentSearchAreas = new string[] { "CMS/Pages" };
            var contentReferences = Enumerable.Empty<ContentReference>();

            foreach (var contentSearchArea in contentSearchAreas)
            {
                var searchProvider = _searchProvidersManager.GetEnabledProvidersByPriority(contentSearchArea, true)
                    .FirstOrDefault();

                if (searchProvider != null)
                {
                    contentReferences = contentReferences.Concat(
                        searchProvider.Search(searchQuery).Select(
                            result => ContentReference.Parse(result.Metadata["Id"])).Distinct());
                }
            }

            
            //return _contentRepository.GetItems(contentReferences, _languageSelectorFactory.AutoDetect(parameters.AllLanguages));
            
            //If you have your search configured correctly you should use the commented row above instead of the one below.
            return _contentRepository.GetChildren<PageData>(PageReference.StartPage, _languageSelectorFactory.AutoDetect(parameters.AllLanguages));

        }
    }
}